@@ -6,7 +6,7 @@ set $*
 case "$1" in
 	button)
 		case "$2" in
-			power)	/sbin/init 0
+			power)	/sbin/shutdown -h now "Power button pressed"
 				;;
 			*)	logger "ACPI action $2 is not defined"
 				;;
