@@ -22,6 +22,7 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/wait.h>
+#include <libgen.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <stdio.h>
@@ -79,19 +80,6 @@ static int safe_write(int fd, const char *buf, int len);
 static char *parse_cmd(const char *cmd, const char *event);
 static int check_escapes(const char *str);
 
-/* helper */
-static int
-path_is_dir(const char *path)
-{
-	struct stat s;
-
-	if (stat(path, &s)) {
-		acpid_log("ERR: stat(\"%s\"): %s\n", path, strerror(errno));
-		return 0;
-	}
-	return S_ISDIR(s.st_mode);
-}
-
 /*
  * read in all the configuration files
  */
@@ -102,6 +90,8 @@ acpid_read_conf(const char *confdir)
 	struct dirent *dirent;
 	char *file = NULL;
 	int nrules = 0;
+	char *basen = NULL;
+	regex_t preg;
 
 	lock_rules();
 
@@ -117,6 +107,7 @@ acpid_read_conf(const char *confdir)
 	while ((dirent = readdir(dir))) {
 		int len;
 		struct rule *r;
+		struct stat stat_buf;
 
 		if (dirent->d_name[0] == '.')
 			continue; /* skip dotfiles */
@@ -130,15 +121,31 @@ acpid_read_conf(const char *confdir)
 		}
 		snprintf(file, len, "%s/%s", confdir, dirent->d_name);
 
-		if (path_is_dir(file)) {
+		/* allow only regular files and symlinks to these */
+		if (stat(file, &stat_buf) != 0) {
+			acpid_log("ERR: stat(%s): %s\n", file, strerror(errno));
+			unlock_rules();
+			return -1;
+		}
+		if (!S_ISREG(stat_buf.st_mode)) {
+			acpid_log("DBG: skipping non-file %s\n", file);
 			free(file);
-			continue; /* skip subdirs */
+			continue;
 		}
 
-		r = parse_file(file);
-		if (r) {
-			enlist_rule(&cmd_list, r);
-			nrules++;
+		/* check for run-parts style filename */
+		basen = basename(file);
+		if (regcomp(&preg, "^[a-zA-Z0-9_-]+$", RULE_REGEX_FLAGS) == 0){
+			if (regexec(&preg, basen, 0, NULL, 0) == 0){
+				r = parse_file(file);
+				if (r) {
+					enlist_rule(&cmd_list, r);
+					nrules++;
+				}
+			} else {
+				acpid_log("DBG: ignoring conf file %s\n", file);
+			}
+
 		}
 		free(file);
 	}
@@ -223,7 +230,7 @@ parse_file(const char *file)
 	}
 
 	/* read each line */
-	while (!feof(fp)) {
+	while (!feof(fp) && !ferror(fp)) {
 		char *p = buf;
 		char key[64];
 		char val[512];
